/* Copyright (C) 2002, by Rupert Paget (rpaget@vision.ee.ethz.ch),
 * all rights reserved.
 *
 * This software is copyrighted as noted below. It may be freely copied,
 * modified, and redistributed, provided that the copyright notice is
 * preserved on all copies.
 *
 * There is no warranty or other guarantee of fitness for this software,
 * it is provided solely "as is". Bug reports or fixes may be sent
 * to the author, who may or may not act on them as he desires.
 *
 * You may not include this software in a commercial program or other
 * software product without the expressed permission of the copyright owner.
 *
 * If you modify this software, you should include a notice giving the
 * name of the person performing the modification, the date of modification,
 * and the reason for such modification.
 */

#include "Neighbourhood.h"

/**************************************************************************/

Neighbourhood::Neighbourhood(int order, bool square, bool pyramid_) :
  x(0), y(0), xP(0), yP(0), pyramid(pyramid_) {
  create(order,square);
  if (pyramid) createP(order-1,square);
  else lengthP = 0;
  index = 0;
  scale = 1;
  lengthTotal = length+lengthP;
}

/**************************************************************************/

Neighbourhood::~Neighbourhood() {
  delete [] x;
  delete [] y;
  delete [] xP;
  delete [] yP;
}

/**************************************************************************/

bool Neighbourhood::neigh(int &xOut, int &yOut) {
  if (index == length) {
    index = 0;
    return false;
  }
  xOut = x[index]*scale;
  yOut = y[index]*scale;
  index++;
  return true;
}

/**************************************************************************/

bool Neighbourhood::neigh(int &xOut, int &yOut, float Neigh, float NeighMin) {
  if ((index == length) || (Neigh > NeighMin)) {
    index = 0;
    return false;
  }
  xOut = x[index]*scale;
  yOut = y[index]*scale;
  index++;
  return true;
}

/**************************************************************************/

bool Neighbourhood::neigh(int &xOut, int &yOut, int &tOut) {
  if (pyramid) {
    if (index == lengthP) {
      index = 0;
      pyramid = false;
      if (index == length) {
	index = 0;
	pyramid = true;
	return false;
      }
      xOut = x[index];
      yOut = y[index];
      tOut = 0;
      index++;
      return true;
    }
    xOut = xP[index];
    yOut = yP[index];
    tOut = 1;
    index++;
    return true;
  }
  if (index == length) {
    index = 0;
    pyramid = true;
    return false;
  }
  xOut = x[index];
  yOut = y[index];
  tOut = 0;
  index++;
  return true;
}

/**************************************************************************/

bool Neighbourhood::neigh(int &xOut, int &yOut, int &tOut,
			  float Neigh, float NeighMin) {
  if (Neigh > NeighMin) {
    index = 0;
    pyramid = true;
    return false;
  }
  return neigh(xOut,yOut,tOut);
}

/**************************************************************************/

int Neighbourhood::getx(int indexIn) const {
  if (indexIn < length) return x[indexIn]*scale;
  else return 0;
}

/**************************************************************************/

int Neighbourhood::gety(int indexIn) const{
  if (indexIn < length) return y[indexIn]*scale;
  else return 0;
}

/**************************************************************************/

void Neighbourhood::create(int order, bool square) {

  /* neighbourhood */
  if(!square){
    int k;
    for(k=1;k*k<=order;k++);
    range = --k;
    x = new int [(2*k+1)*(2*k+1)];
    y = new int [(2*k+1)*(2*k+1)];
    length = 0;
    for(int i= -k;i<=k;i++){
      for(int j= -k;j<=k;j++){
	if((i||j)&&((i*i+j*j)<=order)){
	  x[length]=j;
	  y[length]=i;
	  length++;
	}
      }
    }
  }

  /* neighbourhood square */
  if(square){
    int k = order;
    range = k;
    x = new int [(2*k+1)*(2*k+1)];
    y = new int [(2*k+1)*(2*k+1)];
    length = 0;
    for(int i= -k;i<=k;i++){
      for(int j= -k;j<=k;j++){
	if(i||j){
	  x[length]=j;
	  y[length]=i;
	  length++;
	}
      }
    }
  }
}

/**************************************************************************/

void Neighbourhood::createP(int order, bool square) {

  /* neighbourhood */
  if(!square){
    int k;
    for(k=1;k*k<=order;k++);
    --k;
    xP = new int [(2*k+1)*(2*k+1)];
    yP = new int [(2*k+1)*(2*k+1)];
    lengthP = 0;
    for(int i= -k;i<=k;i++){
      for(int j= -k;j<=k;j++){
	if((i*i+j*j)<=order){
	  xP[lengthP]=j;
	  yP[lengthP]=i;
	  lengthP++;
	}
      }
    }
  }

  /* neighbourhood square */
  if(square){
    int k = order;
    xP = new int [(2*k+1)*(2*k+1)];
    yP = new int [(2*k+1)*(2*k+1)];
    lengthP = 0;
    for(int i= -k;i<=k;i++){
      for(int j= -k;j<=k;j++){
	xP[lengthP]=j;
	yP[lengthP]=i;
	lengthP++;
      }
    }
  }
}

/**************************************************************************/

