/* Copyright (C) 2002, by Rupert Paget (rpaget@vision.ee.ethz.ch),
 * all rights reserved.
 *
 * This software is copyrighted as noted below. It may be freely copied,
 * modified, and redistributed, provided that the copyright notice is
 * preserved on all copies.
 *
 * There is no warranty or other guarantee of fitness for this software,
 * it is provided solely "as is". Bug reports or fixes may be sent
 * to the author, who may or may not act on them as he desires.
 *
 * You may not include this software in a commercial program or other
 * software product without the expressed permission of the copyright owner.
 *
 * If you modify this software, you should include a notice giving the
 * name of the person performing the modification, the date of modification,
 * and the reason for such modification.
 */

#include <iostream>
#include "Pyramid.h"

/**************************************************************************/

Pyramid::Pyramid() {
}

/**************************************************************************/

Pyramid::~Pyramid() {
  /* Input image indexing */
  delete [] imgInIndex;
  delete [] imgInIndexl;
  imgInIndex = 0;
  imgInIndexl = 0;

  /* Output image indexing */
  delete [] imgOutIndex[0];
  delete [] imgOutIndex;
  imgOutIndex = 0;
}

/**************************************************************************/

void Pyramid::load(Option &opt) {
  /* Input training texture */
  imgIn.load(opt.getInputFile(),opt.getLevels());
  rowsIn = imgIn.getHeight();
  colsIn = imgIn.getWidth();

  /* Input mask image */
  maskIn = opt.getMaskIn();
  if (maskIn) {
    imgInMask.load(opt.getInputFileMask(),opt.getLevels());
    if ((rowsIn != imgInMask.getHeight()) ||
	(colsIn != imgInMask.getWidth())) {
      cerr << "Size of mask image does NOT match input texture image";
      cerr << endl;
      exit (1);
    }
  }

  /* Setup Output images */
  rowsOut = opt.getRows();
  colsOut = opt.getCols();
  if (rowsOut == 0) rowsOut = rowsIn;
  if (colsOut == 0) colsOut = colsIn;
  opt.setRows(rowsOut);
  opt.setCols(colsOut);
  
  imgOut.resize(colsOut,rowsOut,opt.getLevels()); //Output texture
  imgProb.resize(colsOut,rowsOut);                //Probability map
  imgProb.clear();
  
  /* Input image indexing */
  /* Dont do this here, as other versions may use higher pyramid levels */
  // imgInIndex = SimilarPixels(imgIn,imgInMask,maskIn,ERROR);

  /* Output image indexing */
  imgOutIndex = new int* [colsOut];
  imgOutIndex[0] = new int [colsOut*rowsOut];
  for (int i = 1; i < colsOut; ++i)
    imgOutIndex[i] = imgOutIndex[i-1]+rowsOut;
}

/**************************************************************************/

bool Pyramid::reduce(Pyramid &pyramid) {
  /* Image sizes */
  if ((rowsIn = (pyramid.rowsIn+1)/2) < 2) return false;
  if ((colsIn = (pyramid.colsIn+1)/2) < 2) return false;
  if ((rowsOut = (pyramid.rowsOut+1)/2) < 2) return false;
  if ((colsOut = (pyramid.colsOut+1)/2) < 2) return false;
  
  /* Input images */
  imgIn.reduce(pyramid.imgIn);
  maskIn = pyramid.maskIn;
  if (maskIn) imgInMask.sample(pyramid.imgInMask);
  
  /* Output images */
  imgOut.resize(colsOut,rowsOut,pyramid.imgOut.getLevels());
  imgProb.resize(colsOut,rowsOut);
  imgProb.clear();
  
  /* Input image indexing */
  /* Dont do this here, as other versions may use higher pyramid levels */
  // imgInIndex = SimilarPixels(imgIn,imgInMask,maskIn,ERROR);
  
  /* Output image indexing */
  imgOutIndex = new int* [colsOut];
  imgOutIndex[0] = new int [colsOut*rowsOut];
  for (int i = 1; i < colsOut; ++i)
    imgOutIndex[i] = imgOutIndex[i-1]+rowsOut;
  
  return true;
}

/**************************************************************************/



