#include "BasicImage.h"

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <iostream>
#include <string>
#include <Magick++.h>

using namespace std;

BasicImage::BasicImage() :
  width(0), height(0), size(0),
  quantLevels(256) {}

BasicImage::BasicImage(int w, int h, int lev) :
  width(w), height(h), size(w*h),
  quantLevels(lev) {}

BasicImage::~BasicImage() {}

bool BasicImage::loadMagick(const string& fname, Magick::Image& magImg) {  
  try {
    // Memory mapped version
    // magImg.read(fname);
    
    // Non memory mapped version
    int fd = open(fname.c_str(),O_RDONLY);
    struct stat sb;
    fstat(fd, &sb);
    char *buf = new char[sb.st_size];
    read(fd,buf,sb.st_size);
    Magick::Blob myblob(buf, sb.st_size);
    magImg.fileName(fname);
    magImg.read(myblob);
    close(fd);
    
  } catch (Magick::Exception &err) {
    cerr << "Caught exception: " << err.what() << endl; 
    return false;
  }

  width = magImg.columns();
  height = magImg.rows();
  size = width*height;

  return true;
}

bool BasicImage::saveMagick(const string& fname, Magick::Image& magImg) const {
  try {
    magImg.write(fname);
  } catch (Magick::Exception &err) {
    cerr << "Caught exception: " << err.what() << endl; 
    return false;
  }
  return true;
}

void BasicImage::writeImageType(ostream& out) const {
  out << "width = " << width << endl;
  out << "height = " << height << endl;
  out << "quant_levels " << quantLevels << endl;
}

