#ifndef _ColourImage_h
#define _ColourImage_h

#include <iostream>
#include <string>
#include <Magick++.h>
#include "BasicImage.h"
#include "GreyImage.h"

using namespace std;

/**
 * Abstraction of an RGBA image.
 * (Actually, only a container for the 4 colour bands.)
 */
class ColourImage : public BasicImage {

public:
  enum Band { RED=0, GREEN=1, BLUE=2 };

protected:
  GreyImage * bands[3]; // array of 4 pointers to bands

public:

  ColourImage(int w = 0, int h = 0, int lev = 256);
  ColourImage(const string& fname, int lev = 256);
  ColourImage(const ColourImage& from);

  virtual ~ColourImage();

  virtual BasicImage *copy() const;

  virtual ColourImage const &operator= (ColourImage const &from);

  virtual void resize(int w, int h);
  virtual void resize(int w, int h, int levels);

  /**
   * Do Burt and Adelson's REDUCE
   * by applying a 5 point Gaussian filter
   */
  ColourImage const &reduce(ColourImage const &from);
  ColourImage const &sample(ColourImage const &from);

  /**
   * Returns a pointer to one colour band.
   * The band has the same parameters (size, quantization, cyclic) as this
   * colour image.
   * The data is not copied!
   */
  inline virtual GreyImage *getBand(int n);

  /**
   * See BasicImage
   */
  virtual bool load(const string& fname);
  virtual bool load(const string& fname, int lev);
  virtual bool save(const string& fname) const;
  virtual void writeImageType(ostream& out) const;
  virtual void shuffle();
  virtual void clear();

protected:
  virtual void copy(ColourImage const &from);
  virtual void destroy();
  virtual void build();
  virtual void separate(const Magick::Blob& bl);
  virtual void combine(const GreyImage& band, int n,
		       Magick::Blob& target) const;
};


// ----- inline stuff -----

GreyImage *ColourImage::getBand(int n) {
  return bands[n];
}

#endif
